﻿using Kivii.Imaging.Filters;
using Kivii.Imaging;
using Kivii.Linq;
using Kivii.Math.Geometry;
using Kivii.Messaging;
using Kivii.Sample.ImageUploader.Entities;
using Kivii.Video;
using Kivii.Video.DirectShow;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Linq;
using static System.Net.Mime.MediaTypeNames;
using Kivii.Canon;

namespace Kivii.Sample.ImageUploader
{
    public partial class FrmMain : Form
    {
        private JsonServiceClient _client = null;

        private FilterInfoCollection videoDevices;
        private VideoCaptureDevice videoDevice;
        private VideoCapabilities[] videoCapabilities;
        private int currentCameraIndex = 0;
        private int cameraQuantity = 0;//相机数量
        private List<DbFile> photos = new List<DbFile>();
        private Report currentReport = null;
        private DbFile currentPhoto = null;
        private CanonForm canonForm = null;

        public FrmMain()
        {
            InitializeComponent();
        }


        #region 界面控件默认值管理
        private void loadDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null) continue;
                    ctl.Text = setting.ControlValue;
                }
            }
        }

        private void setDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null)
                    {
                        setting = new UiSetting();
                        setting.FormName = this.Name;
                        setting.ControlName = ctl.Name;
                        setting.ControlValue = ctl.Text;
                        conn.Insert(setting);
                    }
                    else
                    {
                        setting.ControlValue = ctl.Text;
                        conn.Update(setting);
                    }
                }
            }
        }

        /// <summary>
        /// 输出消息
        /// </summary>
        /// <param name="message">消息内容</param>
        /// <param name="clearMessage">是否清除消息</param>
        private void outputMessage(string message, bool clearMessage = false)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => outputMessage(message, clearMessage)));
                return;
            }
            if (clearMessage) rtbMessage.Clear();
            //if (rtbMessage.Lines.Count() > 500) rtbMessage.Clear();
            //var time = DateTime.Now.ToString("HH:mm:ss");
            //rtbMessage.AppendText($"[{time}]{message}{Environment.NewLine}");
            if (!message.IsNullOrEmpty()) rtbMessage.AppendText($"{message}{Environment.NewLine}");
            //rtbMessage.ScrollToCaret();
        }

        private void initFormCtlEnable(bool notLogin = true)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => initFormCtlEnable(notLogin)));
                return;
            }
            btnLogin.Enabled = notLogin;
            tbxServiceUrl.Enabled = notLogin;
            tbxUserName.Enabled = notLogin;
            tbxPassword.Enabled = notLogin;
            cbxResolution.Enabled = !notLogin;
            cbxTurn.Enabled = !notLogin;
            btnSwitch.Enabled = !notLogin;
            tbxReportId.Enabled = !notLogin;
            btnReportId.Enabled = !notLogin;
            btnShot.Enabled = !notLogin;
            btnDelete.Enabled = !notLogin;
            btnEditor.Enabled = !notLogin;
            btnLogout.Enabled = !notLogin;
            btnCamearTurn.Enabled = !notLogin;
            btnCanon.Enabled = !notLogin;

            if (notLogin)
            {
                btnUpload.Enabled = !notLogin;
                cbxResolution.SelectedIndex = -1;
                //cbxResolution.Items.Clear();
                currentCameraIndex = 0;
                cameraQuantity = 0;
                //gbCamView.Text = "相机视图";
                tbxReportId.Text = string.Empty;
                outputMessage(string.Empty, true);
                //lbCurrentReportId.Text = "__________";
                //lbCurrentSampleName.Text = "__________";
                //lbCurrentBrand.Text = "__________";
                //lbCurrentBatchNumber.Text = "__________";
                //lbCurrentSampleQuantity.Text = "__________";
                ptbPhotoDisplay.Image = null;
                flpPhotos.Controls.Clear();
                currentReport = null;
                currentPhoto = null;
            }
            else
            {
                tbxReportId.Focus();
            }
        }

        private void showCurrentReport()
        {
            if (currentReport == null)
            {
                btnUpload.Enabled = false;
                tbxReportId.Text = string.Empty;
                outputMessage(string.Empty, true);
            }
            else
            {
                btnUpload.Enabled = true;
                tbxReportId.Text = string.Empty;
                var time = DateTime.Now.ToString("HH:mm:ss");
                //outputMessage($"当前时间：{time}");
                Type type = typeof(Report);
                //获取所有属性。
                PropertyInfo[] properties = type.GetProperties();
                foreach (var prop in properties)
                {
                    if (prop.GetDescription().IsNullOrEmpty()) continue;
                    var value = prop.GetValue(currentReport);
                    if (value == null) continue;
                    var msg = $"{prop.GetDescription()}:{value}";
                    outputMessage(msg);

                }
                //lbCurrentReportId.Text = currentReport.ReportId;
                //lbCurrentSampleName.Text = currentReport.SampleName;
                //lbCurrentBrand.Text = currentReport.SampleBrand;
                //lbCurrentBatchNumber.Text = currentReport.SampleBatchNumber;
                //lbCurrentSampleQuantity.Text = currentReport.SampleQuantityUnit;
            }
        }

        private void ImageRotateFlip(RotateFlipType flipType = RotateFlipType.RotateNoneFlipNone)
        {
            if (currentPhoto == null) return;
            if (currentPhoto.Image == null) return;
            int index = 0;
            var photo = photos.FirstOrDefault(o => o.Image == currentPhoto.Image);
            if (photo != null)
            {
                index = photos.IndexOf(photo);
                if (index <= -1) index = 0;
                deleteSnapshot(photo);
            }
            currentPhoto.Image.RotateFlip(flipType);
            var editedPhoto = new DbFile(currentPhoto.Image, currentPhoto.Name);

            addSnapshot(editedPhoto, index);
            showSnapshot(editedPhoto);
        }
        #endregion

        #region 相机控件调用
        private void initCameras()
        {
            try
            {
                // 初始化摄像头
                videoDevices = new FilterInfoCollection(FilterCategory.VideoInputDevice);
                if (videoDevices.Count != 0)
                {
                    // add all devices to combo
                    cameraQuantity = videoDevices.Count;
                    gbCamView.Text = videoDevices[currentCameraIndex].Name;

                    videoDevice = new VideoCaptureDevice(videoDevices[currentCameraIndex].MonikerString);
                    enumeratedSupportedFrameSizes(videoDevice);
                }
                else
                {
                    MessageBox.Show("没有找到视频采集设备");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("无法初始化摄像头：" + ex.Message);
            }
        }

        private void switchCamera()
        {
            if (videoDevice == null) initCameras();
            if (videoDevice == null)
            {
                MessageBox.Show("没有找到视频采集设备");
                return;
            }
            if (videoDevice.IsRunning)
            {
                disconnect();
            }
            if (currentCameraIndex + 1 > cameraQuantity - 1) currentCameraIndex = 0;
            else currentCameraIndex = currentCameraIndex + 1;
            videoDevice = new VideoCaptureDevice(videoDevices[currentCameraIndex].MonikerString);
            gbCamView.Text = videoDevices[currentCameraIndex].Name;
            enumeratedSupportedFrameSizes(videoDevice);
            // 默认选择第一个分辨率
            cbxResolution.SelectedIndex = 0;
        }

        /// <summary>
        /// 列出摄像头支持的分辨率
        /// </summary>
        /// <param name="videoDevice"></param>
        private void enumeratedSupportedFrameSizes(VideoCaptureDevice videoDevice)
        {
            this.Cursor = Cursors.WaitCursor;
            cbxResolution.Items.Clear();
            try
            {
                //videoCapabilities = videoDevice.VideoCapabilities;

                List<VideoCapabilities> tmpVideoCapalilities = new List<VideoCapabilities>();
                // 遍历分辨率列表，并添加到下拉框中
                foreach (VideoCapabilities capabilty in videoDevice.VideoCapabilities)
                {
                    //if (capabilty.FrameSize.Width > 1024) continue;
                    if (capabilty.FrameSize.Width == 1920)
                    {
                        tmpVideoCapalilities.Insert(0, capabilty);
                        cbxResolution.Items.Insert(0, string.Format("{0} x {1}", capabilty.FrameSize.Width, capabilty.FrameSize.Height));
                    }
                    else
                    {
                        tmpVideoCapalilities.Add(capabilty);
                        cbxResolution.Items.Add(string.Format("{0} x {1}", capabilty.FrameSize.Width, capabilty.FrameSize.Height));
                    }
                }
                // 获取摄像头支持的分辨率列表
                videoCapabilities = tmpVideoCapalilities.ToArray();

                // 如果没有支持的分辨率，则添加 "Not supported"
                if (videoCapabilities.Length == 0)
                {
                    cbxResolution.Items.Add("Not supported");
                }
                //// 默认选择第一个分辨率
                //cbxResolution.SelectedIndex = 0;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        private void connect(int selectedIndex = 0)
        {
            if (InvokeRequired)
            {
                Invoke(new Action<int>(connect), selectedIndex);
            }
            else
            {
                if (videoDevice == null) return;
                tbxReportId.Focus();
                if ((videoCapabilities != null) && (videoCapabilities.Length != 0))
                {
                    videoDevice.VideoResolution = videoCapabilities[selectedIndex];
                }
                vspCamView.VideoSource = videoDevice;
                vspCamView.Start();
            }
        }

        private void disconnect()
        {
            if (vspCamView.VideoSource != null)
            {
                // stop video device
                vspCamView.WaitForStop();
                vspCamView.VideoSource = null;
            }
        }

        private void showSnapshot(DbFile photo)
        {
            if (InvokeRequired)
            {
                Invoke(new Action<DbFile>(showSnapshot), photo);
            }
            else
            {
                try
                {
                    //清空 ptbPhotoDisplay 中的图片
                    ptbPhotoDisplay.Image = null;
                    lbDisplay.Text = "";
                    currentPhoto = null;
                    if (photo == null) return;
                    currentPhoto = photo;
                    var snapshot = photo.Image;
                    ptbPhotoDisplay.Image = snapshot;
                    if (photo.Kvid != Guid.Empty) lbDisplay.Text = $"[已上传]{photo.Name}";
                    else lbDisplay.Text = photo.Name;
                    lbDisplay.TextAlign = ContentAlignment.MiddleCenter;

                    // 遍历 flpPhotosTaken 中的控件，将之前选中的 SplitContainer 边框改为默认
                    foreach (Control control in flpPhotos.Controls)
                    {
                        if (control is SplitContainer container)
                        {
                            container.BorderStyle = BorderStyle.None;
                            if (container.Panel1.Controls[0] is PictureBox pic && pic.Image == snapshot)
                            {
                                container.BorderStyle = BorderStyle.Fixed3D;
                            }
                        }
                    }
                    tbxReportId.Focus();

                }
                catch (Exception ex)
                {
                    MessageBox.Show("无法正常显示照片，" + ex.Message);
                }
            }
        }

        private void addSnapshot(DbFile photo,int index=0)
        {
            if (InvokeRequired)
            {
                Invoke(new Action<DbFile,int>(addSnapshot), photo, index);
            }
            else
            {
                if (index < 0) index = 0;
                if (photo == null) return;
                var snapshot = photo.Image;
                if (snapshot == null) return;
                // 创建图片 PictureBox 控件
                var pic = new PictureBox();
                pic.SizeMode = PictureBoxSizeMode.Zoom;
                pic.Dock = DockStyle.Fill;
                pic.Image = snapshot;
                pic.BackColor = Color.LightYellow;
                // 添加图片点击动作
                pic.Click += PictureBox_Click;

                // 创建图片名称的 Label 控件
                Label lbName = new Label();
                string photoName = string.Empty;
                string name = string.Empty;

                photoName = photo.Name.IsNullOrEmpty() ? photo.Name = DateTime.Now.ToString("mmssfff") : photo.Name;

                if (photo.Kvid != Guid.Empty) name = $"[已上传]{photoName}";
                else name = photoName;


                lbName.Text = name;
                lbName.Dock = DockStyle.Fill;
                lbName.TextAlign = ContentAlignment.MiddleCenter;
                lbName.BackColor = Color.LightYellow;

                if(photo.Kvid != Guid.Empty)
                {
                    pic.BackColor = Color.LightGreen;
                    lbName.BackColor = Color.LightGreen;
                }

                // 创建 SplitContainer 控件
                SplitContainer container = new SplitContainer();
                var size = new Size();
                size.Width = flpPhotos.Width - 8;
                size.Height = size.Width / 16 * 9;
                container.Size = size;
                container.Dock = DockStyle.Top;
                container.Orientation = Orientation.Horizontal;
                container.SplitterWidth = 1;
                container.SplitterDistance = 100;
                //container.Panel1.Padding = new Padding(0, 10, 0, 0);
                container.Panel1.Controls.Add(pic);
                container.Panel2.Controls.Add(lbName);
                // 将 SplitContainer 添加到 flpPhotosTaken 中
                flpPhotos.Controls.Add(container);
                // 改变排序，将新增的添加到顶部
                flpPhotos.Controls.SetChildIndex(container, index);

                // 将拍摄的照片对象添加到集合中
                photos.Insert(index, photo);
            }
        }
        private void deleteSnapshot(DbFile photo)
        {
            if (InvokeRequired)
            {
                Invoke(new Action<DbFile>(deleteSnapshot), photo);
            }
            else
            {
                if (photo == null) return;
                var snapshot = photo.Image;

                // 查找并移除对应的 splitContainer
                foreach (Control control in flpPhotos.Controls)
                {
                    if (control is SplitContainer container)
                    {
                        container.BorderStyle = BorderStyle.None;
                        if (container.Panel1.Controls[0] is PictureBox pic && pic.Image == snapshot)
                        {
                            flpPhotos.Controls.Remove(container);
                        }
                    }
                }
                // 将拍摄的照片对象添加到集合中
                photos.Remove(photo);
            }
        }

        private void removeUploadedPhoto()
        {
            if (InvokeRequired)
            {
                Invoke(new Action(removeUploadedPhoto));
            }
            else
            {
                if (photos.IsNullOrEmpty()) return;
                List<DbFile> files = new List<DbFile>();
                files.AddRange(photos);
                foreach (var photo in files)
                {
                    if (photo.Kvid != Guid.Empty) deleteSnapshot(photo);
                }

                Thread.Sleep(50);
                if (!photos.IsNullOrEmpty())
                {
                    showSnapshot(photos.First());
                }
                else
                {
                    showSnapshot(null);
                }

            }
        }
        #endregion

        #region k5业务请求
        private void getReport(string reportId)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            currentReport = null;
            showCurrentReport();
            removeUploadedPhoto();
            if (reportId.IsNullOrEmpty())
            {
                return;
            }
            var request = new ReportRequest();
            request.ReportId = reportId;
            var task = _client.GetAsync(request);
            task.Success(resp =>
            {
                if (resp != null)
                {
                    if (resp.Results.IsNullOrEmpty())
                    {
                        //MessageBox.Show($"未找到此编号：{reportId}");
                        tbxReportId.Text = "";

                        currentReport = new Report();
                        currentReport.ReportId = reportId;
                        currentReport.SampleName = "无样品报告模式";
                        currentReport.SampleModel = "此模式下上传的图片可在公共图片库中查看";
                    }
                    else
                    {
                        if (resp.Results[0].Status != 100)
                        {
                            MessageBox.Show($"此编号{reportId}不在受理状态");
                            tbxReportId.Text = "";
                            currentReport = null;
                        }
                        else
                        {
                            currentReport = resp.Results[0];
                            getDbFiles();
                        }
                    }
                }
                else
                {
                    MessageBox.Show("服务器无响应");
                    tbxReportId.Text = "";
                    currentReport = null;
                }
                showCurrentReport();
            }, true);
            task.Error(err =>
            {
                tbxReportId.Text = "";
                var msg = err.GetResponseStatus().Message;
                MessageBox.Show($"服务器请求错误，{msg}");
                showCurrentReport();
                return;
            }, true);
        }

        private void getDbFiles()
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            if (currentReport == null) return;
            var ownerKvid = currentReport.SampleKvid;
            if (ownerKvid == Guid.Empty) return;
            var request = new DbFileRequest();
            request.OwnerKvid = ownerKvid;
            var task = _client.GetAsync(request);
            task.Success(resp =>
            {
                if (resp != null)
                {
                    if (!resp.Results.IsNullOrEmpty())
                    {
                        var path = Path.Combine(KiviiContext.GetTemporaryFilePath("Kivii"), "Client", "Caches", "Uploader", "Images");
                        if (!Directory.Exists(path)) Directory.CreateDirectory(path);
                        WebClient webClient = new WebClient();

                        foreach (var item in resp.Results)
                        {
                            var downloadPath = $"{tbxServiceUrl.Text}{item.Path}";
                            string filename = Path.Combine(path, item.Name);
                            webClient.DownloadFile(downloadPath, filename);
                            if (!File.Exists(filename)) continue;

                            item.Image = new Bitmap(filename);
                            addSnapshot(item, photos.Count);
                        }

                        Thread.Sleep(50);
                        if (!photos.IsNullOrEmpty())
                        {
                            showSnapshot(photos.First());
                        }
                        else
                        {
                            showSnapshot(null);
                        }
                    }
                }
            }, true);
            task.Error(err =>
            {
                //var msg = err.GetResponseStatus().Message;
                //MessageBox.Show($"服务器请求错误，{msg}");
                //return;
            }, true);
            
        }

        private void deleteDbFile(DbFile photo)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            if (photo == null) return;
            if (photo.Kvid == Guid.Empty) return;

            var request = new DbFileDeleteRequest();
            request.Kvids = new List<Guid>();
            request.Kvids.Add(photo.Kvid);
            try
            {
                _client.Post(request);

            }
            catch (Exception ex)
            {
                var msg = ex.GetResponseStatus().Message;
                MessageBox.Show(msg, "图片删除失败");
            }
        }

        #endregion

        private void FrmMain_Load(object sender, EventArgs e)
        {
            initFormCtlEnable(true);
            loadDefault(tbxServiceUrl, tbxUserName, tbxPassword,cbxTurn);
            initCameras();
            tbxPassword.Focus();
            this.WindowState = FormWindowState.Maximized;
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (_client != null)
            {
                MessageBox.Show("登陆前先退出原连接");
                return;
            }
            btnLogin.Enabled = false;
            if (tbxServiceUrl.Text.IsNullOrEmpty()) return;
            if (tbxUserName.Text.IsNullOrEmpty()) return;
            if (tbxPassword.Text.IsNullOrEmpty()) return;
            _client = new JsonServiceClient(tbxServiceUrl.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                initFormCtlEnable(false);
                setDefault(tbxServiceUrl, tbxUserName, tbxPassword,cbxTurn);
                if (videoDevice == null) initCameras();
                if (videoDevice == null)
                {
                    return;
                }
                Thread.Sleep(100);
                // 默认选择第一个分辨率
                cbxResolution.SelectedIndex = 0;
            }, true);
            task.Error(err =>
            {
                initFormCtlEnable();
                _client.Dispose();
                _client = null;
                var msg = err.GetResponseStatus().Message;
                MessageBox.Show(msg, "Login Error");
            }, true);
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            btnLogout.Enabled = false;

            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                _client.Dispose();
                _client = null;
                Thread.Sleep(100);
                initFormCtlEnable();
                disconnect();
            });
            task.Error(err =>
            {
                btnLogout.Enabled = true;
                MessageBox.Show(err.Message, "Logout Error");
            });
        }

        private void cbxResolution_SelectedIndexChanged(object sender, EventArgs e)
        {
            disconnect();
            Thread.Sleep(100);
            if (cbxResolution.SelectedIndex >= 0) connect(cbxResolution.SelectedIndex);
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            disconnect();
        }

        private void btnSwitch_Click(object sender, EventArgs e)
        {
            switchCamera();
        }

        private void btnShot_Click(object sender, EventArgs e)
        {
            if (videoDevice != null && vspCamView.IsRunning)
            {
                try
                {
                    // 捕获当前帧
                    Bitmap bitmap = (Bitmap)vspCamView.GetCurrentVideoFrame();
                    if (bitmap == null) return;
                    var picName = string.Empty;
                    string reportId = string.Empty;
                    if (currentReport != null)
                    {
                        reportId = currentReport.ReportId;
                    }
                    if (!reportId.IsNullOrEmpty())
                    {
                        picName = reportId + "_";
                    }
                    picName += DateTime.Now.ToString("mmssfff");
                    var photo = new DbFile(bitmap, picName);

                    //if (chbRectify.Checked)
                    //{
                    //    Bitmap temp = photo.Image.Clone() as Bitmap;
                    //    var _counter = new BlobCounter();
                    //    _counter.FilterBlobs = true;
                    //    _counter.MinWidth = photo.Image.Width / 4;
                    //    _counter.MinHeight = photo.Image.Height / 4;

                    //    FiltersSequence seq = new FiltersSequence();
                    //    seq.Add(Grayscale.CommonAlgorithms.BT709);  //First add  grayScaling filter
                    //    seq.Add(new OtsuThreshold()); //Then add binarization(thresholding) filter
                    //    temp = seq.Apply(temp);
                    //    _counter.ProcessImage(temp);
                    //    temp.Dispose();
                    //    var blobs = _counter.GetObjectsInformation();
                    //    if (blobs != null && blobs.Length > 0)
                    //    {
                    //        var corners = PointsCloud.FindQuadrilateralCorners(_counter.GetBlobsEdgePoints(blobs[0]));
                    //        var rect = blobs[0].Rectangle;
                    //        QuadrilateralTransformation quadTransformer = new QuadrilateralTransformation();
                    //        quadTransformer.SourceQuadrilateral = corners; //Set corners for transforming card 
                    //        quadTransformer.AutomaticSizeCalculaton = true;
                    //        photo.Image = quadTransformer.Apply(photo.Image);
                    //        //ResizeBilinear resizer = new ResizeBilinear(rect.Width, rect.Height);
                    //    }
                    //}
                    
                    addSnapshot(photo);
                    Thread.Sleep(50);
                    showSnapshot(photo);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "相机模组未初始化");
                }
                finally
                {
                    var index = cbxTurn.SelectedIndex;
                    if (index <= -1) index = 0;
                    ImageRotateFlip((RotateFlipType)index);
                }

            }
            else
            {
                MessageBox.Show("没有找到视频采集设备");
            }
        }
        private void btnCamearTurn_Click(object sender, EventArgs e)
        {
            if (currentPhoto == null) return;
            if (currentPhoto.Image == null) return;
            ImageRotateFlip(RotateFlipType.Rotate90FlipNone);
        }
        private void PictureBox_Click(object sender, EventArgs e)
        {
            // 获取点击的 PictureBox 中的图片
            PictureBox pictureBox = (PictureBox)sender;

            var photo = photos.FirstOrDefault(o => o.Image == pictureBox.Image);
            if (photo != null) showSnapshot(photo);
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            Bitmap currentPhoto = (Bitmap)ptbPhotoDisplay.Image;
            if (currentPhoto == null) return;
            var photo = photos.FirstOrDefault(o => o.Image == currentPhoto);
            if (photo != null)
            {
                if (photo.Kvid != Guid.Empty)
                {
                    if (MessageBox.Show("当前照片已经上传系统，是否确认从服务器中移除？", "图片删除", MessageBoxButtons.OKCancel) == DialogResult.OK)
                    {
                        deleteDbFile(photo);
                        deleteSnapshot(photo);
                    }
                }
                else deleteSnapshot(photo);
            }
            Thread.Sleep(50);
            if (!photos.IsNullOrEmpty())
            {
                showSnapshot(photos.First());
            }
            else
            {
                showSnapshot(null);
            }
        }

        private void btnReportId_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            //if (tbxReportId.Text.IsNullOrEmpty()) return;
            getReport(tbxReportId.Text);
        }

        private void tbxReportId_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar != 13) return;
            btnReportId_Click(null, null);
        }

        private void tbxPassword_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar != 13) return;
            btnLogin_Click(null, null);
        }

        private void btnUpload_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            if (currentReport == null) return;
            var count = photos.Count;
            if (count == 0) return;
            btnUpload.Enabled = false;
            var path = Path.Combine(KiviiContext.GetTemporaryFilePath("Kivii"), "Client", "Caches", "Uploader", "Images");
            if (!Directory.Exists(path)) Directory.CreateDirectory(path);
            List<DbFile> uploadeds = new List<DbFile>();
            for (int i = count - 1; i >= 0; i--)
            {
                var photo = photos[i];
                if (photo.Image == null) continue;
                if (photo.Kvid != Guid.Empty) continue;

                //string filename = Path.Combine(path, photo.Name + "_" + DateTime.Now.ToString("yyyyMMddHHmmssfff") + ".jpg");
                string filename = Path.Combine(path, photo.Name + ".jpg");
                photo.Image.Save(filename);
                if (!File.Exists(filename)) continue;
                var request = new FileUploadRequest();
                request.OwnerKvid = currentReport.SampleKvid;
                request.FolderPath = "/Samples/Images";
                request.FolderType = "Jst.Lims.Entities.Sample";
                request.Tags = "底页图片";
                try
                {
                    var response = _client.PostFileWithRequest<FileResponse>(new FileInfo(filename), request);
                    if (response.Results != null && response.Results.Count >= 1)
                    {

                        var uploaded = response.Results[0];
                        uploaded.Image = photo.Image;
                        uploadeds.Add(uploaded);
                        if (photo != null) deleteSnapshot(photo);
                    }
                }
                catch (Exception ex)
                {
                    var msg = ex.GetResponseStatus().Message;
                    MessageBox.Show($"当前文件{photo.Name}上传失败,将跳过此文件继续上传剩余文件。{msg}");
                    continue;
                }
            }
            btnUpload.Enabled = true;

            if (!uploadeds.IsNullOrEmpty())
            {
                foreach (var uploaded in uploadeds)
                {
                    addSnapshot(uploaded);
                }
            }
            Thread.Sleep(50);
            if (!photos.IsNullOrEmpty())
            {
                showSnapshot(photos.First());
            }
            else
            {
                showSnapshot(null);
            }

        }

        private void btnEditor_Click(object sender, EventArgs e)
        {
            if (currentPhoto == null) return;
            if (currentPhoto.Kvid != Guid.Empty)
            {
                MessageBox.Show("当前照片已上传，无法编辑");
                return;
            }
            FrmImageEditor frm = new FrmImageEditor();
            Bitmap bmp = currentPhoto.Image;
            frm.backgroundImage = bmp;

            frm.WindowState = FormWindowState.Maximized;
            //frm.Width= bmp.Width;
            //frm.Height = bmp.Height + 120;
            frm.Owner = this;

            frm.ShowDialog();
        }

        public void setBackgroundImage(Bitmap backgroundImage)
        {
            //currentPhoto.Image = backgroundImage;
            int index = 0;
            var photo = photos.FirstOrDefault(o => o.Image == currentPhoto.Image);
            if (photo != null)
            {
                index = photos.IndexOf(photo);
                if (index <= -1) index = 0;
                deleteSnapshot(photo);
            }
            var editedPhoto = new DbFile(backgroundImage, currentPhoto.Name);

            addSnapshot(editedPhoto, index);
            showSnapshot(editedPhoto);
        }

        private void btnEnabled_EnabledChanged(object sender, EventArgs e)
        {
            if(btnShot.Enabled) btnShot.BackColor = Color.White;
            else btnShot.BackColor = SystemColors.ButtonFace;
            if(btnEditor.Enabled) btnEditor.BackColor = Color.White;
            else btnEditor.BackColor = SystemColors.ButtonFace;
            if(btnDelete.Enabled) btnDelete.BackColor = Color.White;
            else btnDelete.BackColor = SystemColors.ButtonFace;
            if (btnUpload.Enabled) btnUpload.BackColor = Color.White;
            else btnUpload.BackColor = SystemColors.ButtonFace;
            if (btnCamearTurn.Enabled) btnCamearTurn.BackColor = Color.White;
            else btnCamearTurn.BackColor = SystemColors.ButtonFace;
            if (btnCanon.Enabled) btnCanon.BackColor = Color.White;
            else btnCanon.BackColor = SystemColors.ButtonFace;

        }

        private void btnCanon_Click(object sender, EventArgs e)
        {
            if (canonForm == null)
            {
                canonForm = new CanonForm();
                canonForm.DownloadCompletely += CanonForm_DownloadCompletely;
            }
            canonForm.ShowDialog();
        }

        private void CanonForm_DownloadCompletely(object sender, EventArgs e)
        {
            var filePath = (string)sender;

            if (filePath.IsNullOrEmpty()) return;

            if(!File.Exists(filePath)) return;

            FileInfo fileInfo = new FileInfo(filePath);
            // 捕获当前帧
            Bitmap bitmap = System.Drawing.Image.FromFile(filePath) as Bitmap;
            if (bitmap == null) return;
            var picName = fileInfo.Name;
            string reportId = string.Empty;
            if (currentReport != null)
            {
                reportId = currentReport.ReportId;
            }
            if (!reportId.IsNullOrEmpty())
            {
                picName = reportId + "_" + picName;
            }
            //picName += DateTime.Now.ToString("mmssfff");
            var photo = new DbFile(bitmap, picName);
            addSnapshot(photo);
            Thread.Sleep(50);
            showSnapshot(photo);
        }

        private void cbxTurn_SelectedIndexChanged(object sender, EventArgs e)
        {
            setDefault(cbxTurn);
        }
    }
}
